@@ -1,8 +1,8 @@
-/* $Id: provider.c,v 1.95 2007/01/07 13:07:00 fpeters Exp $
+/* $Id: provider.c 3291 2007-06-12 14:14:57Z dlaniel $
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
- * Copyright (C) 2004, 2005 Entr'ouvert
+ * Copyright (C) 2004-2007 Entr'ouvert
  * http://lasso.entrouvert.org
  * 
  * Authors: See AUTHORS file in top-level directory.
@@ -411,11 +411,19 @@ get_xmlNode(LassoNode *node, gboolean lasso_dump)
 	xmlNode *xmlnode;
 	LassoProvider *provider = LASSO_PROVIDER(node);
 	char *roles[] = { "None", "SP", "IdP"};
+	char *encryption_mode[] = { "None", "NameId", "Assertion", "Both" };
 
 	xmlnode = parent_class->get_xmlNode(node, lasso_dump);
+
+	/* Save provider role */
 	xmlSetProp(xmlnode, (xmlChar*)"ProviderDumpVersion", (xmlChar*)"2");
-	if (provider->role)
+	if (provider->role) {
 		xmlSetProp(xmlnode, (xmlChar*)"ProviderRole", (xmlChar*)roles[provider->role]);
+	}
+
+	/* Save encryption mode */
+	xmlSetProp(xmlnode, (xmlChar*)"EncryptionMode",
+		(xmlChar*)encryption_mode[provider->private_data->encryption_mode]);
 
 	return xmlnode;
 }
@@ -429,19 +437,43 @@ init_from_xml(LassoNode *node, xmlNode *xmlnode)
 
 	parent_class->init_from_xml(node, xmlnode);
 	
-	if (xmlnode == NULL)
+	if (xmlnode == NULL) {
 		return LASSO_XML_ERROR_OBJECT_CONSTRUCTION_FAILED;
+	}
 
+	/* Load provider role */
 	s = xmlGetProp(xmlnode, (xmlChar*)"ProviderRole");
-	if (s && strcmp((char*)s, "SP") == 0)
+	if (s != NULL && strcmp((char*)s, "SP") == 0) {
 		provider->role = LASSO_PROVIDER_ROLE_SP;
-	if (s && strcmp((char*)s, "IdP") == 0)
+	} else if (s != NULL && strcmp((char*)s, "IdP") == 0) {
 		provider->role = LASSO_PROVIDER_ROLE_IDP;
-	if (s)
+	}
+	if (s != NULL) {
 		xmlFree(s);
+	}
 
-	if (provider->metadata_filename)
+	/* Load encryption mode */
+	s = xmlGetProp(xmlnode, (xmlChar*)"EncryptionMode");
+	if (s != NULL && strcmp((char*)s, "NameId") == 0) {
+		provider->private_data->encryption_mode = LASSO_ENCRYPTION_MODE_NAMEID;
+	} else if (s != NULL && strcmp((char*)s, "Assertion") == 0) {
+		provider->private_data->encryption_mode = LASSO_ENCRYPTION_MODE_ASSERTION;
+	} else if (s != NULL && strcmp((char*)s, "Both") == 0) {
+		provider->private_data->encryption_mode =
+			LASSO_ENCRYPTION_MODE_NAMEID | LASSO_ENCRYPTION_MODE_ASSERTION;
+	}
+	if (s != NULL) {
+		xmlFree(s);
+	}
+
+	/* Load metadata */
+	if (provider->metadata_filename) {
 		lasso_provider_load_metadata(provider, provider->metadata_filename);
+	}
+
+	/* Load signing and encryption public keys */
+	lasso_provider_load_public_key(provider, LASSO_PUBLIC_KEY_SIGNING);
+	lasso_provider_load_public_key(provider, LASSO_PUBLIC_KEY_ENCRYPTION);
 
 	return 0;
 }
@@ -908,9 +940,6 @@ lasso_provider_new_from_dump(const gchar *dump)
 	init_from_xml(LASSO_NODE(provider), xmlDocGetRootElement(doc)); 
 	xmlFreeDoc(doc);
 
-	lasso_provider_load_public_key(provider, LASSO_PUBLIC_KEY_SIGNING);
-	lasso_provider_load_public_key(provider, LASSO_PUBLIC_KEY_ENCRYPTION);
-
 	return provider;
 }
 
